"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const tool_1 = require("../../tool");
class MiddlewarePool {
    constructor(handlers, context) {
        this.handlers = handlers;
        this.context = context;
    }
    wrap(middleware) {
        const handler = async (req, res, nextExpress) => {
            let wasNextCalled = false;
            let wasResCalled = false;
            const next = () => {
                wasNextCalled = true;
                nextExpress();
            };
            const defaultImplementation = res.send;
            res.send = function overrideSend(...args) {
                wasResCalled = true;
                defaultImplementation.apply(this, args);
            };
            await middleware({ req, res, next }, this.context);
            if (!wasNextCalled && !wasResCalled) {
                next();
            }
        };
        return handler;
    }
    async importAndWrap(path) {
        const { default: middleware } = await tool_1.importUncached(path);
        return this.wrap(middleware);
    }
    async load() {
        const array = this.handlers.reduce((prev, handler) => {
            const { request, response } = handler.middlewares;
            const newItems = [...request, ...response]
                .filter((item) => prev.every((existedMiddlewares) => existedMiddlewares !== item));
            return [
                ...prev,
                ...newItems,
            ];
        }, [])
            .map(async (middlewareName) => {
            const mwPath = path_1.join(tool_1.PathsResolverTool.BUILD, `${middlewareName}.js`);
            if (!await tool_1.PathsResolverTool.resolvePath(mwPath)) {
                throw new Error(`You trying to use middleware with path ${mwPath}, but its not exists`);
            }
            return {
                name: middlewareName,
                resolver: await this.importAndWrap(mwPath),
            };
        });
        this.pool = await Promise.all(array);
        return this;
    }
    groupMiddleware(handlerMiddlewareArray) {
        return this.pool
            .filter((middleware) => !handlerMiddlewareArray.every((handlerMw) => handlerMw !== middleware.name))
            .map((middleware) => middleware.resolver);
    }
    getGroupedMiddlewares(handler) {
        const { request, response } = handler.middlewares;
        return {
            request: this.groupMiddleware(request),
            response: this.groupMiddleware(response),
        };
    }
}
exports.default = MiddlewarePool;
//# sourceMappingURL=middleware-pool.js.map