"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadCompilerOptions = exports.mergeCompilerOptions = void 0;
const ts = __importStar(require("typescript"));
const promisified_fs_tool_1 = __importDefault(require("./promisified-fs.tool"));
const paths_resolver_tool_1 = __importDefault(require("./paths-resolver.tool"));
const defaultTsOptions = {
    module: ts.ModuleKind.CommonJS,
    target: ts.ScriptTarget.ES2017,
    esModuleInterop: true,
    strictNullChecks: true,
    emitDecoratorMetadata: true,
    experimentalDecorators: true,
    strict: false,
    outDir: paths_resolver_tool_1.default.BUILD,
    declaration: false,
    sourceMap: false,
    allowJs: true,
};
exports.mergeCompilerOptions = (defaultOptions, loadedOptions) => {
    const { rootDir, outDir, module, target, baseUrl } = loadedOptions, newCompilerOptions = __rest(loadedOptions, ["rootDir", "outDir", "module", "target", "baseUrl"]);
    return Object.assign(Object.assign({}, defaultOptions), newCompilerOptions);
};
exports.loadCompilerOptions = async (path) => {
    const data = await promisified_fs_tool_1.default.readFile(path);
    return JSON.parse(data.toString()).compilerOptions;
};
const getOptions = async () => {
    if (await paths_resolver_tool_1.default.resolvePath(paths_resolver_tool_1.default.TSCONFIG)) {
        try {
            const loadedOptions = await exports.loadCompilerOptions(paths_resolver_tool_1.default.TSCONFIG);
            return exports.mergeCompilerOptions(defaultTsOptions, loadedOptions);
        }
        catch (e) {
            throw new Error(`You are trying to use custom ts compiler options, please check if its correct. ${e}`);
        }
    }
    return defaultTsOptions;
};
exports.default = async (files, rootDir = 'src') => {
    await promisified_fs_tool_1.default.rimraf(paths_resolver_tool_1.default.BUILD);
    const program = ts.createProgram(files, Object.assign(Object.assign({}, await getOptions()), { rootDir }));
    const emitResult = program.emit();
    const allDiagnostics = ts
        .getPreEmitDiagnostics(program)
        .concat(emitResult.diagnostics);
    allDiagnostics.forEach((diagnostic) => {
        if (diagnostic.file) {
            const { line, character } = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
            const message = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
            throw new Error(`${diagnostic.file.fileName} (${line + 1},${character + 1}): ${message}`);
        }
    });
};
//# sourceMappingURL=compiler.tool.js.map